@@ -8,7 +8,19 @@ from typing import Iterable
 import math
 
 
-def in_parallel(entries: list, max_parallelism=None, num_stages:int = 1):
+def in_parallel(entries: list, max_parallelism=None, num_stages:int = 1, return_exceptions=False):
+    """
+    Creates a parallism of tasks to be run concurrently
+
+    pipeline = in_parallel([a.send(i) for i in range(100)]) 
+
+    :param entries: the tasks to run
+    :param optional max_parallelism: the maximum number of tasks to run in parallel at a time
+    :param optional num_stages: the number of sub-pipelines to break this parallelism into.  Used to scale larger parallelisms.
+    :param optional return_exceptions: if True then tasks that raise exceptions will not cause an aggregated exception to be thrown but instead will appear in the results
+    :return: a pipeline
+    """
+
     if num_stages > 1:
         # split up a group such [[1,2,3,4,5,6]] into inline pipelines each with a subset of the group
         # i.e. [[p[1,2], p[3,4], p[5,6]]] followed by a flatten to allow for better distribution
@@ -20,9 +32,9 @@ def in_parallel(entries: list, max_parallelism=None, num_stages:int = 1):
         if len(stages) > 1:
             per_stage_pipeline = [PipelineBuilder().append_group(stage, max_parallelism=per_stage_max_parallelism) for stage in stages]
             group = [PipelineBuilder().append(builtin.run_pipeline.to_task(args=pipeline.inline())) for pipeline in per_stage_pipeline] 
-            return PipelineBuilder().append_group(group, max_parallelism=max_parallelism).continue_with(builtin.flatten_results)
+            return PipelineBuilder().append_group(group, max_parallelism=max_parallelism, return_exceptions=return_exceptions).continue_with(builtin.flatten_results)
 
-    return PipelineBuilder().append_group(entries, max_parallelism)
+    return PipelineBuilder().append_group(entries, max_parallelism, return_exceptions=return_exceptions)
 
 
 class PipelineBuilder(ProtobufSerialisable):
@@ -31,12 +43,12 @@ class PipelineBuilder(ProtobufSerialisable):
 
     :param optional pipeline: list of initial pipeline entries e.g. from another builder
     """
-
     def __init__(self, pipeline: list = None):
         self._pipeline = [] if pipeline is None else pipeline
         self._builder_id = _gen_id()
         self._inline = False
         self._initial_args = None
+        self._initial_kwargs = None
         self._initial_state = None
 
     @property
@@ -77,15 +89,16 @@ class PipelineBuilder(ProtobufSerialisable):
         other._pipeline.extend(self._pipeline)
         return self
 
-    def append_group(self, pipelines: Iterable['PipelineBuilder'], max_parallelism=None) -> 'PipelineBuilder':
+    def append_group(self, pipelines: Iterable['PipelineBuilder'], max_parallelism=None, return_exceptions=False) -> 'PipelineBuilder':
         """
         Appends tasks from another pipeline builder into a new in_parallel group inside this one
 
         :param other: the other pipeline builder
         :param option max_parallelism: the maximum number of tasks to invoke in parallel for this group 
+        :param option return_exceptions: if True then tasks that raise exceptions will not cause an aggregated exception to be thrown but instead will appear in the results
         :return: the builder
         """
-        group = Group(_gen_id(), max_parallelism=max_parallelism)
+        group = Group(_gen_id(), max_parallelism=max_parallelism, return_exceptions=return_exceptions)
 
         for pipeline in pipelines:
             pipeline._add_to_group(group)
@@ -211,10 +224,6 @@ class PipelineBuilder(ProtobufSerialisable):
         args = self.validate().to_proto(serialiser)
         kwargs = {}
 
-        # send a single argument by itself instead of wrapped inside a tuple
-        if _is_tuple(args) and len(args) == 1:
-            args = args[0]
-
         task_request = TaskRequest(id=task_id, type=task_type, uid=_gen_id())
         args_and_kwargs = serialiser.serialise_args_and_kwargs(args, kwargs)
         serialiser.serialise_request(task_request, args_and_kwargs)
@@ -252,6 +261,7 @@ class PipelineBuilder(ProtobufSerialisable):
         return _Pipeline(self._pipeline, 
             inline=self._inline, 
             initial_args=self._initial_args,
+            initial_kwargs=self._initial_kwargs,
             initial_state=self._initial_state,
             is_fruitful=is_fruitful,
             serialiser=serialiser,
@@ -270,15 +280,23 @@ class PipelineBuilder(ProtobufSerialisable):
 
         return self
 
-    def with_initial(self, args=None, state=None) ->  'PipelineBuilder':
+    def with_initial(self, args=Ellipsis, kwargs=Ellipsis, state=Ellipsis) ->  'PipelineBuilder':
         """
-        Optionally sets the initial args and state to be passed to the initial tasks(s) in this pipeline
-        :param option args: arguments
+        Optionally sets the initial args kwargs and state to be passed to the initial tasks(s) in this pipeline
+        :param option args: arguments as tuple or TupleOfAny
         :param option state: state
+        :param option kwargs: keyword arguments as dict or MapStringToAny
         :return: the builder
         """
-        self._initial_args = args
-        self._initial_state = state
+        if args != Ellipsis:
+            self._initial_args = args
+
+        if kwargs != Ellipsis:
+            self._initial_kwargs = kwargs
+
+        if state != Ellipsis:
+            self._initial_state = state
+            
         return self
 
     def inline(self, is_inline=True) ->  'PipelineBuilder':
@@ -351,14 +369,14 @@ class PipelineBuilder(ProtobufSerialisable):
         if pipeline_proto.inline:
             builder = builder.inline()
 
-        args, state = None, None
         if pipeline_proto.HasField('initial_args'):
-            args = pipeline_proto.initial_args
+            builder.with_initial(args=pipeline_proto.initial_args)
+
+        if pipeline_proto.HasField('initial_kwargs'):
+            builder.with_initial(kwargs=pipeline_proto.initial_kwargs)
 
         if pipeline_proto.HasField('initial_state'):
-            state = pipeline_proto.initial_state
-
-        builder = builder.with_initial(args, state)
+            builder.with_initial(state=pipeline_proto.initial_state)
 
         return builder
 
