@@ -31,7 +31,8 @@ class BeginPipelineHandler(PipelineMessageHandler):
         context.pipeline_state.status.value = TaskStatus.RUNNING
         context.pipeline_state.pipeline.CopyFrom(pipeline.to_proto())
         context.pipeline_state.is_fruitful = pipeline.is_fruitful
-        context.pipeline_state.task_state.CopyFrom(pack_any(self.serialiser.to_proto(state)))
+        context.pipeline_state.task_state.CopyFrom(pack_any(self.serialiser.to_proto(state)))  # state of task calling this pipeline
+        context.pipeline_state.last_task_state.CopyFrom(context.pipeline_state.pipeline.initial_state) # state of last task called within this pipeline
         context.pipeline_state.invocation_id = _gen_id()
 
         if caller_id is not None:
@@ -62,20 +63,23 @@ class BeginPipelineHandler(PipelineMessageHandler):
         else:
             # else get initial tasks(s) to call - might be single start of chain task or a group of tasks to call in parallel
             tasks, max_parallelism, slice = self.graph.get_initial_tasks()
+            initial_state, initial_args, initial_kwargs = None, None, None
 
             # send initial state to each initial task if we have some
-            if context.pipeline_state.pipeline.HasField('initial_state'):                
-                task_state = unpack_any(context.pipeline_state.pipeline.initial_state, known_proto_types=[])
-            else:
-                task_state = None
+            if context.pipeline_state.pipeline.HasField('initial_state'):
+                initial_state = unpack_any(context.pipeline_state.pipeline.initial_state, known_proto_types=[])
+
+            # or the current task state if this pipeline is inline
+            elif context.pipeline_state.pipeline.inline:
+                initial_state = unpack_any(context.pipeline_state.task_state, known_proto_types=[])
 
             # send initial arguments to each initial task if we have them
             if context.pipeline_state.pipeline.HasField('initial_args'):
-                args = unpack_any(context.pipeline_state.pipeline.initial_args, known_proto_types=[])
+                initial_args = unpack_any(context.pipeline_state.pipeline.initial_args, known_proto_types=[])
 
-                for task in tasks:
-                    task_args_and_kwargs = self._serialiser.to_args_and_kwargs(task.request)
-                    task.request = self._serialiser.merge_args_and_kwargs(args, task_args_and_kwargs)
+            # send initial kwargs to each initial task if we have them
+            if context.pipeline_state.pipeline.HasField('initial_kwargs'):
+                initial_kwargs = context.pipeline_state.pipeline.initial_kwargs
 
             # if we skipped over empty group(s) then make sure we pass empty array to next task (result of in_parallel([]) is intuitively [])
             if slice > 0:
@@ -84,7 +88,13 @@ class BeginPipelineHandler(PipelineMessageHandler):
                     task.request = self._serialiser.serialise_args_and_kwargs(([]), kwargs)
 
             # split into tasks to call now and those to defer if max parallelism is exceeded
-            self.submitter.submit_tasks(context, tasks, task_state=task_state, max_parallelism=max_parallelism)
+            self.submitter.submit_tasks(
+                context, 
+                tasks, 
+                initial_args=initial_args, 
+                initial_kwargs=initial_kwargs,
+                initial_state=initial_state,
+                max_parallelism=max_parallelism)
 
             # break
             return False, message
