@@ -15,10 +15,13 @@ class ContinuePipelineHandler(PipelineMessageHandler):
                 and context.pipeline_state.status.value in [TaskStatus.RUNNING, TaskStatus.PAUSED] \
                     and isinstance(message, (TaskResult, TaskException))
 
-    async def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], pipeline: '_Pipeline', **kwargs):
+    async def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], pipeline, **kwargs):
         task_result_or_exception = message
         caller_uid = message.uid
 
+        # update last known task state in case we need to cancel later and call a finally task passing through the current state
+        context.pipeline_state.last_task_state.CopyFrom(task_result_or_exception.state)          
+
         # mark pipeline step as complete
         if not self.graph.mark_task_complete(caller_uid, task_result_or_exception):
             return False, task_result_or_exception
@@ -37,13 +40,18 @@ class ContinuePipelineHandler(PipelineMessageHandler):
         current_step, next_step, group, empty_group = self.graph.get_next_step_in_pipeline(caller_uid)
 
         # if this task is part group then we need to record the results so we can aggregate later
+        group_is_complete = False
         if group is not None:
+            group_is_complete = group.is_complete()
             self.result_aggregator.add_result(context, caller_uid, task_result_or_exception)
 
             # once the group is complete aggregate the results
-            if group.is_complete():
+            if group_is_complete:
                 task_result_or_exception = self.result_aggregator.aggregate(context, group)
 
+                # update last known task state in case we need to cancel later and call a finally task passing through the current state
+                context.pipeline_state.last_task_state.CopyFrom(task_result_or_exception.state)   
+
                 # pause the pipeline if this completed group is a wait
                 if group.is_wait:
                     await pipeline.pause(context)
@@ -53,8 +61,12 @@ class ContinuePipelineHandler(PipelineMessageHandler):
                 await pipeline.pause(context)
 
         # if we got an exception then the next step is the finally_task if there is one (or none otherwise)
-        if isinstance(task_result_or_exception, TaskException):
-            next_step = self.graph.try_get_finally_task(caller_uid)
+        # but only once the group is complete (if we are in a group)
+        if isinstance(task_result_or_exception, TaskException):        
+            if group is not None and group_is_complete:
+                next_step = self.graph.try_get_finally_task(caller_uid) if group_is_complete else None
+            else:
+                next_step = self.graph.try_get_finally_task(caller_uid)
 
         # else if we came across an empty group between this task and next_entry then our result must be an empty array []
         # as we cannot call an empty group but can synthesise the result (remembering to pass through state)
@@ -64,8 +76,7 @@ class ContinuePipelineHandler(PipelineMessageHandler):
 
         # turn next step into remainder of tasks to call
         if isinstance(next_step, Task):
-            tasks = [next_step]
-            max_parallelism = 1
+            tasks, max_parallelism = [next_step], 1
 
             if next_step.is_finally:
                 # record the result of the task prior to the finally task so we can return it once the finally task completes
@@ -89,11 +100,10 @@ class ContinuePipelineHandler(PipelineMessageHandler):
                 await pipeline.events.notify_pipeline_status_changed(context, context.pipeline_state.pipeline, context.pipeline_state.status.value)
 
             elif isinstance(task_result_or_exception, TaskException):
-                if group is None or group.is_complete():
+                if group is None or group_is_complete:
                     # else if have an exception then we failed but waiting for any parallel tasks in the group to complete first
                     context.pipeline_state.status.value = TaskStatus.FAILED
                     await pipeline.events.notify_pipeline_status_changed(context, context.pipeline_state.pipeline, context.pipeline_state.status.value)
 
         # continue
         return True, task_result_or_exception
-    
\ No newline at end of file
